@@ -14,7 +14,7 @@ if not modules then modules = { } end modules ['buff-ver'] = {
 -- todo: update to match context scite lexing
 
 local type, next, rawset, rawget, setmetatable, getmetatable, tonumber = type, next, rawset, rawget, setmetatable, getmetatable, tonumber
-local format, lower, upper,match, find, sub = string.format, string.lower, string.upper, string.match, string.find, string.sub
+local lower, upper,match, find, sub = string.lower, string.upper, string.match, string.find, string.sub
 local splitlines = string.splitlines
 local concat = table.concat
 local C, P, R, S, V, Carg, Cc, Cs = lpeg.C, lpeg.P, lpeg.R, lpeg.S, lpeg.V, lpeg.Carg, lpeg.Cc, lpeg.Cs
@@ -33,6 +33,8 @@ local context              = context
 local commands             = commands
 local implement            = interfaces.implement
 
+local formatters           = string.formatters
+
 local tabtospace           = utilities.strings.tabtospace
 local variables            = interfaces.variables
 local settings_to_array    = utilities.parsers.settings_to_array
@@ -40,11 +42,13 @@ local variables            = interfaces.variables
 local findfile             = resolvers.findfile
 local addsuffix            = file.addsuffix
 
-local v_auto               = variables.auto
 local v_yes                = variables.yes
+local v_no                 = variables.no
 local v_last               = variables.last
 local v_all                = variables.all
 local v_absolute           = variables.absolute
+----- v_inline             = variables.inline  -- not !
+----- v_display            = variables.display -- not !
 
 -- beware, all macros have an argument:
 
@@ -123,7 +127,8 @@ local signal = "\000"
 visualizers.signal        = signal
 visualizers.signalpattern = P(signal)
 
-local functions = { __index = {
+local functions = {
+    __index = {
         emptyline = f_emptyline,
         newline   = f_newline,
         default   = f_default,
@@ -138,7 +143,8 @@ local functions = { __index = {
 local handlers = { }
 
 function visualizers.newhandler(name,data)
-    local tname, tdata = type(name), type(data)
+    local tname = type(name)
+    local tdata = type(data)
     if tname == "table" then -- (data)
         setmetatable(name,getmetatable(name) or functions)
         return name
@@ -236,12 +242,13 @@ function visualizers.load(name)
     name = lower(name)
     if rawget(specifications,name) == nil then
         name = lower(name)
-        local texname = findfile(format("buff-imp-%s.mkiv",name))
-        local luaname = findfile(format("buff-imp-%s.lua" ,name))
+        local impname = "buff-imp-"..name
+        local texname = findfile(addsuffix(impname,"mkiv"))
+        local luaname = findfile(addsuffix(impname,"lua"))
         if texname == "" or luaname == "" then
             -- assume a user specific file
             luaname = findfile(addsuffix(name,"mkiv"))
-            texname = findfile(addsuffix(name,"lua" ))
+            texname = findfile(addsuffix(name,"lua"))
         end
         if texname == "" or luaname == "" then
             if trace_visualize then
@@ -251,7 +258,7 @@ function visualizers.load(name)
             if trace_visualize then
                 report_visualizers("loading visualizer %a",name)
             end
-            lua.registercode(luaname)
+            lua.registercode(luaname) -- only used here, end up in format
             context.input(texname)
         end
         if rawget(specifications,name) == nil then
@@ -266,14 +273,15 @@ function visualizers.register(name,specification)
         report_visualizers("registering visualizer %a",name)
     end
     specifications[name] = specification
-    local parser, handler = specification.parser, specification.handler
-    local displayparser = specification.display or parser
-    local inlineparser = specification.inline or parser
-    local isparser = is_lpeg(parser)
+    local parser         = specification.parser
+    local handler        = specification.handler
+    local displayparser  = specification.display or parser
+    local inlineparser   = specification.inline  or parser
+    local isparser       = is_lpeg(parser)
     local start, stop
     if isparser then
         start = makepattern(handler,"start",patterns.alwaysmatched)
-        stop = makepattern(handler,"stop",patterns.alwaysmatched)
+        stop  = makepattern(handler,"stop", patterns.alwaysmatched)
     end
     if handler then
         if isparser then
@@ -374,7 +382,7 @@ function visualizers.registerescapepattern(name,befores,afters,normalmethod,esca
                     name,i,before,after,processor or "default")
             end
             before = P(before) * space_pattern
-            after = space_pattern * P(after)
+            after  = space_pattern * P(after)
             local action
             if processor then
                 action = function(s) apply_processor(processor,s) end
@@ -440,7 +448,7 @@ function visualizers.registerescapecommand(name,token,normalmethod,escapecommand
         end
         token = P(token)
         local notoken = hack((1 - token)^1)
-        local cstoken = name_pattern * space_pattern
+        local cstoken = Cs(name_pattern * (space_pattern/""))
         escapepattern = (
             (token / "")
           * (cstoken / (escapecommand or texcommand))
@@ -451,15 +459,16 @@ function visualizers.registerescapecommand(name,token,normalmethod,escapecommand
     return escapepattern
 end
 
-local escapedvisualizers = { }
+local escapedvisualizers  = { }
+local f_escapedvisualizer = formatters["%s : %s"]
 
 local function visualize(content,settings) -- maybe also method in settings
     if content and content ~= "" then
         local method = lower(settings.method or "default")
-        local m
+        local m = specifications[method] or specifications.default
         local e = settings.escape
-        if e and e ~= "" then
-            local newname = format("%s : %s",method,e)
+        if e and e ~= "" and not m.handler.noescape then
+            local newname = f_escapedvisualizer(method,e)
             local newspec = specifications[newname]
             if newspec then
                 m = newspec
@@ -483,9 +492,10 @@ local function visualize(content,settings) -- maybe also method in settings
                         end
                     end
                 end
-                local oldvisualizer = specifications[method] or specifications.default
-                local oldparser = oldvisualizer.direct
-                local newparser
+                local oldm       = m
+                local oldparser  = oldm.direct
+                local newhandler = oldm.handler
+                local newparser  = oldm.parser -- nil
                 if starts[1] and stops[1] ~= "" then
                     newparser = visualizers.registerescapepattern(newname,starts,stops,oldparser,nil,processors)
                 elseif starts[1] then
@@ -495,7 +505,7 @@ local function visualize(content,settings) -- maybe also method in settings
                 end
                 m = visualizers.register(newname, {
                     parser  = newparser,
-                    handler = oldvisualizer.handler,
+                    handler = newhandler,
                 })
             end
         else
@@ -555,7 +565,7 @@ local beginline = C(patterns.beginline)   * CargOne / f_beginline
 local anything  = C(patterns.somecontent) * CargOne / f_default
 
 ----- verbosed  = (space + newline * (emptyline^0) * beginline + anything)^0
-local verbosed  = (space + newline * (emptyline^0) * beginline + emptyline + newline + anything)^0
+local verbosed  = (space + newline * (emptyline^0) * beginline + newline * emptyline + newline + anything)^0
 
 local function write(s,settings) -- bad name
     lpegmatch(verbosed,s,1,settings or false)
@@ -607,7 +617,12 @@ end
 local onlyspaces = S(" \t\f\n\r")^0 * P(-1)
 
 local function getstrip(lines,first,last)
-    local first, last = first or 1, last or #lines
+    if not first then
+        first = 1
+    end
+    if not last then
+        last = #lines
+    end
     for i=first,last do
         local li = lines[i]
         if #li == 0 or lpegmatch(onlyspaces,li) then
@@ -729,12 +744,14 @@ end
 
 local function filter(lines,settings) -- todo: inline or display in settings
     local strip = settings.strip
-    if strip and strip ~= "" then
+ -- if strip and strip == "" then
+    if strip ~= v_no and strip ~= false then
         lines = realign(lines,strip)
     end
-    local line, n = 0, 0
-    local first, last, m = getstrip(lines)
+    local line  = 0
+    local n     = 0
     local range = settings.range
+    local first, last, m = getstrip(lines)
     if range then
         first, last = getrange(lines,first,last,range)
         first, last = getstrip(lines,first,last)
@@ -790,8 +807,8 @@ local nospace   = space^1/""
 local endstring = P(-1)
 
 local compactors = {
-    [v_all]      = Cs((backslash * (1-backslash-space)^1 * nospace * (endstring + fences) + 1)^0),
-    [v_absolute] = Cs((backslash * (1-symbols  -space)^1 * nospace * (symbols+backslash) + 1) ^0),
+    [v_all]      = Cs((backslash * (1-backslash-space)^1 * nospace * (endstring+fences) + 1)^0),
+    [v_absolute] = Cs((backslash * (1-symbols  -space)^1 * nospace * (symbols +backslash) + 1) ^0),
     [v_last]     = Cs((space^1   * endstring/"" + 1)^0),
 }
 
@@ -801,7 +818,7 @@ local function typestring(settings)
         local compact   = settings.compact
         local compactor = compact and compactors[compact]
         if compactor then
-            content = lpegmatch(compactor,content)
+            content = lpegmatch(compactor,content) or content
         end
      -- content = decodecomment(content)
      -- content = dotabs(content,settings)
@@ -837,7 +854,7 @@ implement {
     actions   = typestring,
     arguments = {
         {
-            { "data"    },
+            { "data" },
             { "tab"     },
             { "method"  },
             { "compact" },
@@ -847,6 +864,50 @@ implement {
     }
 }
 
+-- implement {
+--     name      = "type_x",
+--     actions   = typestring,
+--     arguments = {
+--         {
+--             { "data", "verbatim" },
+--             { "tab"     },
+--             { "method"  },
+--             { "compact" },
+--             { "nature"  },
+--             { "escape"  },
+--         }
+--     }
+-- }
+
+-- local function typestring_y(settings)
+--     local content = tex.toks[settings.n]
+--     if content and content ~= "" then
+--         local compact   = settings.compact
+--         local compactor = compact and compactors[compact]
+--         if compactor then
+--             content = lpegmatch(compactor,content)
+--         end
+--      -- content = decodecomment(content)
+--      -- content = dotabs(content,settings)
+--         visualize(content,checkedsettings(settings,"inline"))
+--     end
+-- end
+
+-- implement {
+--     name      = "type_y",
+--     actions   = typestring_y,
+--     arguments = {
+--         {
+--             { "n", "integer" },
+--             { "tab"     },
+--             { "method"  },
+--             { "compact" },
+--             { "nature"  },
+--             { "escape"  },
+--         }
+--     }
+-- }
+
 implement {
     name      = "processbuffer",
     actions   = processbuffer,
