@@ -192,6 +192,7 @@ insert(characters.ranges,{
     end,
 })
 
+
 local blocks = allocate {
     ["adlam"]                                      = { first = 0x1E900, last = 0x1E95F,             description = "Adlam" },
     ["aegeannumbers"]                              = { first = 0x10100, last = 0x1013F,             description = "Aegean Numbers" },
@@ -297,12 +298,14 @@ local blocks = allocate {
  -- ["digitsthai"]                                 = { first = 0x00E50, last = 0x00E59, math = true },
  -- ["digitstibetan"]                              = { first = 0x00F20, last = 0x00F29, math = true },
     ["dingbats"]                                   = { first = 0x02700, last = 0x027BF,              description = "Dingbats" },
-    ["dogra"]                                      = { first = 0x11800, last = 0x1184F,             description = "Dogra" },
+    ["dogra"]                                      = { first = 0x11800, last = 0x1184F,              description = "Dogra" },
     ["dominotiles"]                                = { first = 0x1F030, last = 0x1F09F,              description = "Domino Tiles" },
     ["duployan"]                                   = { first = 0x1BC00, last = 0x1BC9F,              description = "Duployan" },
     ["earlydynasticcuneiform"]                     = { first = 0x12480, last = 0x1254F,              description = "Early Dynastic Cuneiform" },
+    ["egyptianhieroglyphformatcontrols"]           = { first = 0x13430, last = 0x1343F,              description = "Egyptian Hieroglyph Format Controls" },
     ["egyptianhieroglyphs"]                        = { first = 0x13000, last = 0x1342F,              description = "Egyptian Hieroglyphs" },
     ["elbasan"]                                    = { first = 0x10500, last = 0x1052F,              description = "Elbasan" },
+    ["elymaic"]                                    = { first = 0x10FE0, last = 0x10FFF,              description = "Elymaic" },
     ["emoticons"]                                  = { first = 0x1F600, last = 0x1F64F,              description = "Emoticons" },
     ["enclosedalphanumerics"]                      = { first = 0x02460, last = 0x024FF,              description = "Enclosed Alphanumerics" },
     ["enclosedalphanumericsupplement"]             = { first = 0x1F100, last = 0x1F1FF,              description = "Enclosed Alphanumeric Supplement" },
@@ -316,7 +319,7 @@ local blocks = allocate {
     ["geometricshapes"]                            = { first = 0x025A0, last = 0x025FF, math = true, description = "Geometric Shapes" },
     ["geometricshapesextended"]                    = { first = 0x1F780, last = 0x1F7FF,              description = "Geometric Shapes Extended" },
     ["georgian"]                                   = { first = 0x010A0, last = 0x010FF, otf="geor",  description = "Georgian" },
-    ["georgianextended"]                           = { first = 0x01C90, last = 0x01CBF,             description = "Georgian Extended" },
+    ["georgianextended"]                           = { first = 0x01C90, last = 0x01CBF,              description = "Georgian Extended" },
     ["georgiansupplement"]                         = { first = 0x02D00, last = 0x02D2F, otf="geor",  description = "Georgian Supplement" },
     ["glagolitic"]                                 = { first = 0x02C00, last = 0x02C5F, otf="glag",  description = "Glagolitic" },
     ["glagoliticsupplement"]                       = { first = 0x1E000, last = 0x1E02F,              description = "Glagolitic Supplement" },
@@ -325,7 +328,7 @@ local blocks = allocate {
     ["greekandcoptic"]                             = { first = 0x00370, last = 0x003FF, otf="grek",  description = "Greek and Coptic" },
     ["greekextended"]                              = { first = 0x01F00, last = 0x01FFF, otf="grek",  description = "Greek Extended" },
     ["gujarati"]                                   = { first = 0x00A80, last = 0x00AFF, otf="gujr",  description = "Gujarati" },
-    ["gunjalagondi"]                               = { first = 0x11D60, last = 0x11DAF,             description = "Gunjala Gondi" },
+    ["gunjalagondi"]                               = { first = 0x11D60, last = 0x11DAF,              description = "Gunjala Gondi" },
     ["gurmukhi"]                                   = { first = 0x00A00, last = 0x00A7F, otf="guru",  description = "Gurmukhi" },
     ["halfwidthandfullwidthforms"]                 = { first = 0x0FF00, last = 0x0FFEF,              description = "Halfwidth and Fullwidth Forms" },
     ["hangulcompatibilityjamo"]                    = { first = 0x03130, last = 0x0318F, otf="jamo",  description = "Hangul Compatibility Jamo" },
@@ -333,7 +336,7 @@ local blocks = allocate {
     ["hanguljamoextendeda"]                        = { first = 0x0A960, last = 0x0A97F,              description = "Hangul Jamo Extended-A" },
     ["hanguljamoextendedb"]                        = { first = 0x0D7B0, last = 0x0D7FF,              description = "Hangul Jamo Extended-B" },
     ["hangulsyllables"]                            = { first = 0x0AC00, last = 0x0D7AF, otf="hang",  description = "Hangul Syllables" },
-    ["hanifirohingya"]                             = { first = 0x10D00, last = 0x10D3F,             description = "Hanifi Rohingya" },
+    ["hanifirohingya"]                             = { first = 0x10D00, last = 0x10D3F,              description = "Hanifi Rohingya" },
     ["hanunoo"]                                    = { first = 0x01720, last = 0x0173F, otf="hano",  description = "Hanunoo" },
     ["hatran"]                                     = { first = 0x108E0, last = 0x108FF,              description = "Hatran" },
     ["hebrew"]                                     = { first = 0x00590, last = 0x005FF, otf="hebr",  description = "Hebrew" },
@@ -343,7 +346,7 @@ local blocks = allocate {
     ["ideographicdescriptioncharacters"]           = { first = 0x02FF0, last = 0x02FFF,              description = "Ideographic Description Characters" },
     ["ideographicsymbolsandpunctuation"]           = { first = 0x16FE0, last = 0x16FFF,              description = "Ideographic Symbols and Punctuation" },
     ["imperialaramaic"]                            = { first = 0x10840, last = 0x1085F,              description = "Imperial Aramaic" },
-    ["indicsiyaqnumbers"]                          = { first = 0x1EC70, last = 0x1ECBF,             description = "Indic Siyaq Numbers" },
+    ["indicsiyaqnumbers"]                          = { first = 0x1EC70, last = 0x1ECBF,              description = "Indic Siyaq Numbers" },
     ["inscriptionalpahlavi"]                       = { first = 0x10B60, last = 0x10B7F,              description = "Inscriptional Pahlavi" },
     ["inscriptionalparthian"]                      = { first = 0x10B40, last = 0x10B5F,              description = "Inscriptional Parthian" },
     ["ipaextensions"]                              = { first = 0x00250, last = 0x002AF,              description = "IPA Extensions" },
@@ -402,7 +405,7 @@ local blocks = allocate {
     ["lydian"]                                     = { first = 0x10920, last = 0x1093F,              description = "Lydian" },
     ["mahajani"]                                   = { first = 0x11150, last = 0x1117F,              description = "Mahajani" },
     ["mahjongtiles"]                               = { first = 0x1F000, last = 0x1F02F,              description = "Mahjong Tiles" },
-    ["makasar"]                                    = { first = 0x11EE0, last = 0x11EFF,             description = "Makasar" },
+    ["makasar"]                                    = { first = 0x11EE0, last = 0x11EFF,              description = "Makasar" },
     ["malayalam"]                                  = { first = 0x00D00, last = 0x00D7F, otf="mlym",  description = "Malayalam" },
     ["mandaic"]                                    = { first = 0x00840, last = 0x0085F, otf="mand",  description = "Mandaic" },
     ["manichaean"]                                 = { first = 0x10AC0, last = 0x10AFF,              description = "Manichaean" },
@@ -410,8 +413,8 @@ local blocks = allocate {
     ["masaramgondi"]                               = { first = 0x11D00, last = 0x11D5F,              description = "Masaram Gondi" },
     ["mathematicalalphanumericsymbols"]            = { first = 0x1D400, last = 0x1D7FF, math = true, description = "Mathematical Alphanumeric Symbols" },
     ["mathematicaloperators"]                      = { first = 0x02200, last = 0x022FF, math = true, description = "Mathematical Operators" },
-    ["mayannumerals"]                              = { first = 0x1D2E0, last = 0x1D2FF,             description = "Mayan Numerals" },
-    ["medefaidrin"]                                = { first = 0x16E40, last = 0x16E9F,             description = "Medefaidrin" },
+    ["mayannumerals"]                              = { first = 0x1D2E0, last = 0x1D2FF,              description = "Mayan Numerals" },
+    ["medefaidrin"]                                = { first = 0x16E40, last = 0x16E9F,              description = "Medefaidrin" },
     ["meeteimayek"]                                = { first = 0x0ABC0, last = 0x0ABFF,              description = "Meetei Mayek" },
     ["meeteimayekextensions"]                      = { first = 0x0AAE0, last = 0x0AAFF,              description = "Meetei Mayek Extensions" },
     ["mendekikakui"]                               = { first = 0x1E800, last = 0x1E8DF,              description = "Mende Kikakui" },
@@ -435,11 +438,13 @@ local blocks = allocate {
     ["myanmarextendeda"]                           = { first = 0x0AA60, last = 0x0AA7F,              description = "Myanmar Extended-A" },
     ["myanmarextendedb"]                           = { first = 0x0A9E0, last = 0x0A9FF,              description = "Myanmar Extended-B" },
     ["nabataean"]                                  = { first = 0x10880, last = 0x108AF,              description = "Nabataean" },
+    ["nandinagari"]                                = { first = 0x119A0, last = 0x119FF,              description = "Nandinagari" },
     ["newa"]                                       = { first = 0x11400, last = 0x1147F,              description = "Newa" },
     ["newtailue"]                                  = { first = 0x01980, last = 0x019DF,              description = "New Tai Lue" },
     ["nko"]                                        = { first = 0x007C0, last = 0x007FF, otf="nko",   description = "NKo" },
     ["numberforms"]                                = { first = 0x02150, last = 0x0218F,              description = "Number Forms" },
     ["nushu"]                                      = { first = 0x1B170, last = 0x1B2FF,              description = "Nushu" },
+    ["nyiakengpuachuehmong"]                       = { first = 0x1E100, last = 0x1E14F,              description = "Nyiakeng Puachue Hmong" },
     ["ogham"]                                      = { first = 0x01680, last = 0x0169F, otf="ogam",  description = "Ogham" },
     ["olchiki"]                                    = { first = 0x01C50, last = 0x01C7F,              description = "Ol Chiki" },
     ["oldhungarian"]                               = { first = 0x10C80, last = 0x10CFF,              description = "Old Hungarian" },
@@ -447,7 +452,7 @@ local blocks = allocate {
     ["oldnortharabian"]                            = { first = 0x10A80, last = 0x10A9F,              description = "Old North Arabian" },
     ["oldpermic"]                                  = { first = 0x10350, last = 0x1037F,              description = "Old Permic" },
     ["oldpersian"]                                 = { first = 0x103A0, last = 0x103DF, otf="xpeo",  description = "Old Persian" },
-    ["oldsogdian"]                                 = { first = 0x10F00, last = 0x10F2F,             description = "Old Sogdian" },
+    ["oldsogdian"]                                 = { first = 0x10F00, last = 0x10F2F,              description = "Old Sogdian" },
     ["oldsoutharabian"]                            = { first = 0x10A60, last = 0x10A7F,              description = "Old South Arabian" },
     ["oldturkic"]                                  = { first = 0x10C00, last = 0x10C4F,              description = "Old Turkic" },
     ["opticalcharacterrecognition"]                = { first = 0x02440, last = 0x0245F,              description = "Optical Character Recognition" },
@@ -455,6 +460,7 @@ local blocks = allocate {
     ["ornamentaldingbats"]                         = { first = 0x1F650, last = 0x1F67F,              description = "Ornamental Dingbats" },
     ["osage"]                                      = { first = 0x104B0, last = 0x104FF,              description = "Osage" },
     ["osmanya"]                                    = { first = 0x10480, last = 0x104AF, otf="osma",  description = "Osmanya" },
+    ["ottomansiyaqnumbers"]                        = { first = 0x1ED00, last = 0x1ED4F,              description = "Ottoman Siyaq Numbers" },
     ["pahawhhmong"]                                = { first = 0x16B00, last = 0x16B8F,              description = "Pahawh Hmong" },
     ["palmyrene"]                                  = { first = 0x10860, last = 0x1087F,              description = "Palmyrene" },
     ["paucinhau"]                                  = { first = 0x11AC0, last = 0x11AFF,              description = "Pau Cin Hau" },
@@ -478,7 +484,8 @@ local blocks = allocate {
     ["sinhala"]                                    = { first = 0x00D80, last = 0x00DFF, otf="sinh",  description = "Sinhala" },
     ["sinhalaarchaicnumbers"]                      = { first = 0x111E0, last = 0x111FF,              description = "Sinhala Archaic Numbers" },
     ["smallformvariants"]                          = { first = 0x0FE50, last = 0x0FE6F,              description = "Small Form Variants" },
-    ["sogdian"]                                    = { first = 0x10F30, last = 0x10F6F,             description = "Sogdian" },
+    ["smallkanaextension"]                         = { first = 0x1B130, last = 0x1B16F,              description = "Small Kana Extension" },
+    ["sogdian"]                                    = { first = 0x10F30, last = 0x10F6F,              description = "Sogdian" },
     ["sorasompeng"]                                = { first = 0x110D0, last = 0x110FF,              description = "Sora Sompeng" },
     ["soyombo"]                                    = { first = 0x11A50, last = 0x11AAF,              description = "Soyombo" },
     ["spacingmodifierletters"]                     = { first = 0x002B0, last = 0x002FF,              description = "Spacing Modifier Letters" },
@@ -496,6 +503,7 @@ local blocks = allocate {
     ["supplementaryprivateuseareab"]               = { first = 0x100000,last = 0x10FFFF,             description = "Supplementary Private Use Area-B" },
     ["suttonsignwriting"]                          = { first = 0x1D800, last = 0x1DAAF,              description = "Sutton SignWriting" },
     ["sylotinagri"]                                = { first = 0x0A800, last = 0x0A82F, otf="sylo",  description = "Syloti Nagri" },
+    ["symbolsandpictographsextendeda"]             = { first = 0x1FA70, last = 0x1FAFF,              description = "Symbols and Pictographs Extended-A" },
     ["syriac"]                                     = { first = 0x00700, last = 0x0074F, otf="syrc",  description = "Syriac" },
     ["syriacsupplement"]                           = { first = 0x00860, last = 0x0086F,              description = "Syriac Supplement" },
     ["tagalog"]                                    = { first = 0x01700, last = 0x0171F, otf="tglg",  description = "Tagalog" },
@@ -507,6 +515,7 @@ local blocks = allocate {
     ["taixuanjingsymbols"]                         = { first = 0x1D300, last = 0x1D35F,              description = "Tai Xuan Jing Symbols" },
     ["takri"]                                      = { first = 0x11680, last = 0x116CF,              description = "Takri" },
     ["tamil"]                                      = { first = 0x00B80, last = 0x00BFF, otf="taml",  description = "Tamil" },
+    ["tamilsupplement"]                            = { first = 0x11FC0, last = 0x11FFF,              description = "Tamil Supplement" },
     ["tangut"]                                     = { first = 0x17000, last = 0x187FF,              description = "Tangut" },
     ["tangutcomponents"]                           = { first = 0x18800, last = 0x18AFF,              description = "Tangut Components" },
     ["telugu"]                                     = { first = 0x00C00, last = 0x00C7F, otf="telu",  description = "Telugu" },
@@ -544,6 +553,7 @@ local blocks = allocate {
     ["variationselectorssupplement"]               = { first = 0xE0100, last = 0xE01EF,              description = "Variation Selectors Supplement" },
     ["vedicextensions"]                            = { first = 0x01CD0, last = 0x01CFF,              description = "Vedic Extensions" },
     ["verticalforms"]                              = { first = 0x0FE10, last = 0x0FE1F,              description = "Vertical Forms" },
+    ["wancho"]                                     = { first = 0x1E2C0, last = 0x1E2FF,              description = "Wancho" },
     ["warangciti"]                                 = { first = 0x118A0, last = 0x118FF,              description = "Warang Citi" },
     ["yijinghexagramsymbols"]                      = { first = 0x04DC0, last = 0x04DFF, otf="yi",    description = "Yijing Hexagram Symbols" },
     ["yiradicals"]                                 = { first = 0x0A490, last = 0x0A4CF, otf="yi",    description = "Yi Radicals" },
@@ -1516,7 +1526,7 @@ do
     )^1)
 
     local function load()
-        local name = resolvers.find_file("char-emj.lua")
+        local name = resolvers.findfile("char-emj.lua")
         local data = name and name ~= "" and dofile(name) or { }
         local hash = { }
         for d, c in next, data do
