@@ -6,6 +6,8 @@ if not modules then modules = { } end modules ['lpdf-col'] = {
     license   = "see context related readme files"
 }
 
+-- slants also page ?
+
 local type, next, tostring, tonumber = type, next, tostring, tonumber
 local char, byte, format, gsub, rep, gmatch = string.char, string.byte, string.format, string.gsub, string.rep, string.gmatch
 local settings_to_array, settings_to_numbers = utilities.parsers.settings_to_array, utilities.parsers.settings_to_numbers
@@ -23,7 +25,7 @@ local registrations           = backends.pdf.registrations
 
 local nodepool                = nodes.nuts.pool
 local register                = nodepool.register
-local pdfpageliteral          = nodepool.pdfpageliteral
+local pageliteral             = nodepool.pageliteral
 
 local pdfconstant             = lpdf.constant
 local pdfdictionary           = lpdf.dictionary
@@ -58,18 +60,18 @@ local setmetatableindex       = table.setmetatableindex
 
 local c_transparency          = pdfconstant("Transparency")
 
-local f_gray   = formatters["%.3F g %.3F G"]
-local f_rgb    = formatters["%.3F %.3F %.3F rg %.3F %.3F %.3F RG"]
-local f_cmyk   = formatters["%.3F %.3F %.3F %.3F k %.3F %.3F %.3F %.3F K"]
+local f_gray   = formatters["%.3N g %.3N G"]
+local f_rgb    = formatters["%.3N %.3N %.3N rg %.3N %.3N %.3N RG"]
+local f_cmyk   = formatters["%.3N %.3N %.3N %.3N k %.3N %.3N %.3N %.3N K"]
 local f_spot   = formatters["/%s cs /%s CS %s SCN %s scn"]
 local f_tr     = formatters["Tr%s"]
-local f_cm     = formatters["q %.6F %.6F %.6F %.6F %.6F %.6F cm"]
-local f_effect = formatters["%s Tc %s w %s Tr"]
+local f_cm     = formatters["q %.6N %.6N %.6N %.6N %.6N %.6N cm"]
+local f_effect = formatters["%s Tc %s w %s Tr"] -- %.6N ?
 local f_tr_gs  = formatters["/Tr%s gs"]
-local f_num_1  = tostring
-local f_num_2  = formatters["%s %s"]
-local f_num_3  = formatters["%s %s %s"]
-local f_num_4  = formatters["%s %s %s %s"]
+local f_num_1  = formatters["%.3N %.3N"]
+local f_num_2  = formatters["%.3N %.3N"]
+local f_num_3  = formatters["%.3N %.3N %.3N"]
+local f_num_4  = formatters["%.3N %.3N %.3N %.3N"]
 
 local report_color = logs.reporter("colors","backend")
 
@@ -122,26 +124,26 @@ lpdf.registerpagefinalizer(addpagegroup,3,"pagegroup")
 -- color injection
 
 function nodeinjections.rgbcolor(r,g,b)
-    return register(pdfpageliteral(f_rgb(r,g,b,r,g,b)))
+    return register(pageliteral(f_rgb(r,g,b,r,g,b)))
 end
 
 function nodeinjections.cmykcolor(c,m,y,k)
-    return register(pdfpageliteral(f_cmyk(c,m,y,k,c,m,y,k)))
+    return register(pageliteral(f_cmyk(c,m,y,k,c,m,y,k)))
 end
 
 function nodeinjections.graycolor(s) -- caching 0/1 does not pay off
-    return register(pdfpageliteral(f_gray(s,s)))
+    return register(pageliteral(f_gray(s,s)))
 end
 
 function nodeinjections.spotcolor(n,f,d,p)
     if type(p) == "string" then
         p = gsub(p,","," ") -- brr misuse of spot
     end
-    return register(pdfpageliteral(f_spot(n,n,p,p)))
+    return register(pageliteral(f_spot(n,n,p,p)))
 end
 
 function nodeinjections.transparency(n)
-    return register(pdfpageliteral(f_tr_gs(n)))
+    return register(pageliteral(f_tr_gs(n)))
 end
 
 -- a bit weird but let's keep it here for a while
@@ -160,7 +162,7 @@ function nodeinjections.effect(effect,stretch,rulethickness)
     -- always, no zero test (removed)
     rulethickness = bp * rulethickness
     effect = effects[effect] or effects['normal']
-    return register(pdfpageliteral(f_effect(stretch,rulethickness,effect))) -- watch order
+    return register(pageliteral(f_effect(stretch,rulethickness,effect))) -- watch order
 end
 
 -- spot- and indexcolors
@@ -335,7 +337,9 @@ local function registersomeindexcolor(name,noffractions,names,p,colorspace,range
         colorspace,
         pdfreference(n),
     }
-    local vector, set, n = { }, { }, #values
+    local vector = { }
+    local set    = { }
+    local n      = #values
     for i=255,0,-1 do
         for j=1,n do
             set[j] = format("%02X",round(values[j]*i))
@@ -420,7 +424,8 @@ function codeinjections.setfigurecolorspace(data,figure)
         local ref = indexcolorref(color)
         if ref then
             figure.colorspace = ref
-            data.used.color = color
+            data.used.color    = color
+            data.used.colorref = ref
         end
     end
 end
@@ -512,13 +517,21 @@ local function lpdfcolor(model,ca,default) -- todo: use gray when no color
                 local s = cv[2]
                 return f_gray(s,s)
             elseif model == 3 then
-                local r, g, b = cv[3], cv[4], cv[5]
+                local r = cv[3]
+                local g = cv[4]
+                local b = cv[5]
                 return f_rgb(r,g,b,r,g,b)
             elseif model == 4 then
-                local c, m, y, k = cv[6],cv[7],cv[8],cv[9]
+                local c = cv[6]
+                local m = cv[7]
+                local y = cv[8]
+                local k = cv[9]
                 return f_cmyk(c,m,y,k,c,m,y,k)
             else
-                local n,f,d,p = cv[10],cv[11],cv[12],cv[13]
+                local n = cv[10]
+                local f = cv[11]
+                local d = cv[12]
+                local p = cv[13]
                 if type(p) == "string" then
                     p = gsub(p,","," ") -- brr misuse of spot
                 end
@@ -617,7 +630,7 @@ function lpdf.colorvalues(model,ca,default)
             return cv[3], cv[4], cv[5]
         elseif model == 4 then
             return cv[6], cv[7], cv[8], cv[9]
-        elseif model == 4 then
+        elseif model == 5 then
             return cv[13]
         end
     else
@@ -715,39 +728,39 @@ do
     local pdfcolor        = lpdf.color
     local pdftransparency = lpdf.transparency
 
-    local f_slant = formatters["q 1 0 %.6F 1 0 0 cm"]
+    local f_slant = formatters["q 1 0 %N 1 0 0 cm"]
 
  -- local fillcolors = {
- --     red        = { "pdf", "origin", "1 0 0 rg" },
- --     green      = { "pdf", "origin", "0 1 0 rg" },
- --     blue       = { "pdf", "origin", "0 0 1 rg" },
- --     gray       = { "pdf", "origin", ".5 g" },
- --     black      = { "pdf", "origin", "0 g" },
- --     palered    = { "pdf", "origin", "1 .75 .75 rg" },
- --     palegreen  = { "pdf", "origin", ".75 1 .75 rg" },
- --     paleblue   = { "pdf", "origin", ".75 .75 1 rg" },
- --     palegray   = { "pdf", "origin", ".75 g" },
+ --     red        = { "pdf", "page", "1 0 0 rg" },
+ --     green      = { "pdf", "page", "0 1 0 rg" },
+ --     blue       = { "pdf", "page", "0 0 1 rg" },
+ --     gray       = { "pdf", "page", ".5 g" },
+ --     black      = { "pdf", "page", "0 g" },
+ --     palered    = { "pdf", "page", "1 .75 .75 rg" },
+ --     palegreen  = { "pdf", "page", ".75 1 .75 rg" },
+ --     paleblue   = { "pdf", "page", ".75 .75 1 rg" },
+ --     palegray   = { "pdf", "page", ".75 g" },
  -- }
  --
  -- local strokecolors = {
- --     red        = { "pdf", "origin", "1 0 0 RG" },
- --     green      = { "pdf", "origin", "0 1 0 RG" },
- --     blue       = { "pdf", "origin", "0 0 1 RG" },
- --     gray       = { "pdf", "origin", ".5 G" },
- --     black      = { "pdf", "origin", "0 G" },
- --     palered    = { "pdf", "origin", "1 .75 .75 RG" },
- --     palegreen  = { "pdf", "origin", ".75 1 .75 RG" },
- --     paleblue   = { "pdf", "origin", ".75 .75 1 RG" },
- --     palegray   = { "pdf", "origin", ".75 G" },
+ --     red        = { "pdf", "page", "1 0 0 RG" },
+ --     green      = { "pdf", "page", "0 1 0 RG" },
+ --     blue       = { "pdf", "page", "0 0 1 RG" },
+ --     gray       = { "pdf", "page", ".5 G" },
+ --     black      = { "pdf", "page", "0 G" },
+ --     palered    = { "pdf", "page", "1 .75 .75 RG" },
+ --     palegreen  = { "pdf", "page", ".75 1 .75 RG" },
+ --     paleblue   = { "pdf", "page", ".75 .75 1 RG" },
+ --     palegray   = { "pdf", "page", ".75 G" },
  -- }
  --
  -- backends.pdf.tables.vfspecials = allocate { -- todo: distinguish between glyph and rule color
  --
- --     red          = { "pdf", "origin", "1 0 0 rg 1 0 0 RG" },
- --     green        = { "pdf", "origin", "0 1 0 rg 0 1 0 RG" },
- --     blue         = { "pdf", "origin", "0 0 1 rg 0 0 1 RG" },
- --     gray         = { "pdf", "origin", ".75 g .75 G" },
- --     black        = { "pdf", "origin", "0 g 0 G" },
+ --     red          = { "pdf", "page", "1 0 0 rg 1 0 0 RG" },
+ --     green        = { "pdf", "page", "0 1 0 rg 0 1 0 RG" },
+ --     blue         = { "pdf", "page", "0 0 1 rg 0 0 1 RG" },
+ --     gray         = { "pdf", "page", ".75 g .75 G" },
+ --     black        = { "pdf", "page", "0 g 0 G" },
  --
  --  -- rulecolors   = fillcolors,
  --  -- fillcolors   = fillcolors,
@@ -770,7 +783,7 @@ do
 
     local c_cache = setmetatableindex(function(t,m)
         local v = setmetatableindex(function(t,c)
-            local p = { "pdf", "origin", "q " .. pdfcolor(m,c) }
+            local p = { "pdf", "page", "q " .. pdfcolor(m,c) }
             t[c] = p
             return p
         end)
@@ -784,7 +797,7 @@ do
         local p = pdftransparency(transparency)
         local v = setmetatableindex(function(t,colormodel)
             local v = setmetatableindex(function(t,color)
-                local v = { "pdf", "origin", "q " .. pdfcolor(colormodel,color) .. " " .. p }
+                local v = { "pdf", "page", "q " .. pdfcolor(colormodel,color) .. " " .. p }
                 t[color] = v
                 return v
             end)
@@ -808,8 +821,8 @@ do
     backends.pdf.tables.vfspecials = allocate { -- todo: distinguish between glyph and rule color
 
         startcolor = startcolor,
-     -- stopcolor  = { "pdf", "origin", "0 g 0 G Q" },
-        stopcolor  = { "pdf", "origin", "Q" },
+     -- stopcolor  = { "pdf", "page", "0 g 0 G Q" },
+        stopcolor  = { "pdf", "page", "Q" },
 
         startslant = startslant,
         stopslant  = { "pdf", "origin", "Q" },
