@@ -33,13 +33,12 @@ if not modules then modules = { } end modules ['math-map'] = {
 -- plus add them to the regular vectors below so that they honor \it etc
 
 local type, next = type, next
-local floor, div = math.floor, math.div
-local merged = table.merged
+local merged, sortedhash = table.merged, table.sortedhash
 local extract = bit32.extract
 
 local allocate            = utilities.storage.allocate
 
-local otffeatures         = fonts.constructors.newfeatures("otf")
+local otffeatures         = fonts.constructors.features.otf
 local registerotffeature  = otffeatures.register
 
 local setmetatableindex   = table.setmetatableindex
@@ -118,6 +117,7 @@ mathematics.gaps = allocate {
     [0x1D4AD] = 0x0211B, -- ℛ script R
     [0x1D4BA] = 0x0212F, -- ℯ script e
     [0x1D4BC] = 0x0210A, -- ℊ script g
+ -- [0x1D4C1] = 0x02113, -- exception: liter
     [0x1D4C4] = 0x02134, -- ℴ script o
     [0x1D506] = 0x0212D, -- ℭ fraktur C
     [0x1D50B] = 0x0210C, -- ℌ fraktur H
@@ -164,6 +164,10 @@ local function toupper (n) local t = { } for i=0,25 do t[0x00041+i] = n+i end re
 local function tolower (n) local t = { } for i=0,25 do t[0x00061+i] = n+i end return t end
 local function tovector(t)                                                    return t end
 
+-- how about 0x2A (ast)    cq. 0x2217
+--           0x2D (hyphen) cq. 0x2212
+--           0x3A (colon)  cq. 0x2236
+
 local regular_tf = {
     digits    = todigit(0x00030),
     ucletters = toupper(0x00041),
@@ -553,10 +557,10 @@ mathematics.mapremap  = mathremap
 local boldmap         = allocate { }
 mathematics.boldmap   = boldmap
 
--- all math (a bit of redundancy here)
+-- all math (a bit of redundancy here) (sorted for tracing)
 
-for alphabet, styles in next, alphabets do -- per 9/6/2011 we also have attr for missing
-    for style, data in next, styles do
+for alphabet, styles in sortedhash(alphabets) do -- per 9/6/2011 we also have attr for missing
+    for style, data in sortedhash(styles) do
      -- let's keep the long names (for tracing)
         local n = #mathremap + 1
         local d = {
@@ -647,7 +651,7 @@ end
 
 implement {
     name      = "setmathattribute",
-    arguments = { "string", "string" },
+    arguments = "2 strings",
     actions   = function(alphabet,style)
         local data = alphabets[alphabet] or regular
         data = data[style] or data.tf
@@ -737,17 +741,17 @@ function mathematics.remapalphabets(char,mathalphabet,mathgreek)
         if not isgreek[char] then
             -- nothing needed
         elseif islcgreek[char] then
-            local lc = extract(mathgreek,4,4)
+            local lc = extract(mathgreek,4,4) --  (mathgreek >> 4) & ~(-1 << 4)
             if lc > 1 then
                 mathalphabet = remapgreek(mathalphabet,lc,"lowercase",char)
             end
         elseif isucgreek[char] then
-            local uc = extract(mathgreek,0,4)
+            local uc = extract(mathgreek,0,4) --  (mathgreek >> 0) & ~(-1 << 4)
             if uc > 1 then
                 mathalphabet = remapgreek(mathalphabet,uc,"uppercase",char)
             end
         elseif issygreek[char] then
-            local sy = extract(mathgreek,8,4)
+            local sy = extract(mathgreek,8,4) --  (mathgreek >> 8) & ~(-1 << 4)
             if sy > 1 then
                 mathalphabet = remapgreek(mathalphabet,sy,"symbol",char)
             end
