@@ -6,54 +6,34 @@ if not modules then modules = { } end modules ['mlib-ctx'] = {
     license   = "see context related readme files",
 }
 
--- for the moment we have the scanners here but they migh tbe moved to
--- the other modules
-
 local type, tostring = type, tostring
 local format, concat = string.format, table.concat
 local settings_to_hash = utilities.parsers.settings_to_hash
+local formatters = string.formatters
 
-local report_metapost = logs.reporter("metapost")
+local report_metapost = logs.reporter ("metapost")
+local status_metapost = logs.messenger("metapost")
 
-local starttiming        = statistics.starttiming
-local stoptiming         = statistics.stoptiming
+local starttiming     = statistics.starttiming
+local stoptiming      = statistics.stoptiming
 
-local mplib              = mplib
+local trace_graphic   = false
 
-metapost                 = metapost or {}
-local metapost           = metapost
+trackers.register("metapost.graphics",
+    function(v) trace_graphic = v end
+);
 
-local context            = context
+local mplib            = mplib
 
-local setters            = tokens.setters
-local setmacro           = setters.macro
-local implement          = interfaces.implement
+metapost               = metapost or { }
+local metapost         = metapost
+local context          = context
 
-local v_no               = interfaces.variables.no
+local setters          = tokens.setters
+local setmacro         = setters.macro
+local implement        = interfaces.implement
 
-metapost.defaultformat   = "metafun"
-metapost.defaultinstance = "metafun"
-metapost.defaultmethod   = "default"
-
-local function setmpsformat(specification)
-    local instance = specification.instance
-    local format   = specification.format
-    local method   = specification.method
-    if not instance or instance == "" then
-        instance = metapost.defaultinstance
-        specification.instance = instance
-    end
-    if not format or format == "" then
-        format = metapost.defaultformat
-        specification.format = format
-    end
-    if not method or method == "" then
-        method = metapost.defaultmethod
-        specification.method = method
-    end
-    specification.mpx = metapost.format(instance,format,method)
-    return specification
-end
+local v_no             = interfaces.variables.no
 
 local extensiondata    = metapost.extensiondata or storage.allocate { }
 metapost.extensiondata = extensiondata
@@ -87,14 +67,10 @@ function metapost.getextensions(instance,state)
     end
 end
 
--- function commands.getmpextensions(instance,state)
---     context(metapost.getextensions(instance,state))
--- end
-
 implement {
     name      = "setmpextensions",
     actions   = metapost.setextensions,
-    arguments = { "string", "string" }
+    arguments = "2 strings",
 }
 
 implement {
@@ -103,10 +79,8 @@ implement {
     arguments = "string"
 }
 
-local report_metapost = logs.reporter ("metapost")
-local status_metapost = logs.messenger("metapost")
-
 local patterns = {
+    CONTEXTLMTXMODE > 0 and "meta-imp-%s.mkxl" or "",
     "meta-imp-%s.mkiv",
     "meta-imp-%s.tex",
     -- obsolete:
@@ -115,10 +89,8 @@ local patterns = {
 }
 
 local function action(name,foundname)
+    commands.loadlibrary(name,foundname,false)
     status_metapost("library %a is loaded",name)
-    context.startreadingfile()
-    context.input(foundname)
-    context.stopreadingfile()
 end
 
 local function failure(name)
@@ -139,7 +111,7 @@ implement {
     end
 }
 
--- metapost.variables  = { } -- to be stacked
+-- metapost.variables = { } -- to be stacked
 
 implement {
     name      = "mprunvar",
@@ -179,7 +151,7 @@ implement {
 
 implement {
     name      = "mprunset",
-    arguments = { "string", "string" },
+    arguments = "2 strings",
     actions   = function(name,connector)
         local value = metapost.variables[name]
         if value ~= nil then
@@ -199,7 +171,41 @@ implement {
 -- or just move the scanners to pps
 
 function metapost.graphic(specification)
-    metapost.graphic_base_pass(setmpsformat(specification))
+    metapost.pushformat(specification)
+    metapost.graphic_base_pass(specification)
+    metapost.popformat()
+end
+
+function metapost.startgraphic(t)
+    if not t then
+        t = { }
+    end
+    if not t.instance then
+        t.instance = metapost.defaultinstance
+    end
+    if not t.format then
+        t.format = metapost.defaultformat
+    end
+    if not t.method then
+        t.method = metapost.defaultmethod
+    end
+    t.data = { }
+    return t
+end
+
+function metapost.stopgraphic(t)
+    if t then
+        t.data = concat(t.data or { },"\n")
+        if trace_graphic then
+            report_metapost("\n"..t.data.."\n")
+        end
+        metapost.graphic(t)
+    end
+end
+
+function metapost.tographic(t,f,s,...)
+    local d = t.data
+    d[#d+1] = s and formatters[f](s,...) or f
 end
 
 implement {
@@ -238,9 +244,10 @@ implement {
     arguments = "string",
 }
 
+-- this has to become a codeinjection
+
 function metapost.getclippath(specification) -- why not a special instance for this
-    setmpsformat(specification)
-    local mpx = specification.mpx
+    local mpx  = metapost.pushformat(specification)
     local data = specification.data or ""
     if mpx and data ~= "" then
         starttiming(metapost)
@@ -259,7 +266,10 @@ function metapost.getclippath(specification) -- why not a special instance for t
             result = metapost.filterclippath(result)
         end
         stoptiming(metapost)
+        metapost.pushformat()
         return result
+    else
+        metapost.pushformat()
     end
 end
 
@@ -295,7 +305,14 @@ end
 implement {
     name      = "mpsetclippath",
     actions   = function(specification)
-        setmacro("MPclippath",metapost.theclippath(specification),"global")
+        local p = specification.data and metapost.theclippath(specification)
+        if not p or p == "" then
+            local b = number.dimenfactors.bp
+            local w = b * (specification.width or 0)
+            local h = b * (specification.height or 0)
+            p = formatters["0 0 m %.6N 0 l %.6N %.6N l 0 %.6N l"](w,w,h,h)
+        end
+        setmacro("MPclippath",p,"global")
     end,
     arguments = {
         {
@@ -307,27 +324,23 @@ implement {
             { "inclusions" },
             { "method" },
             { "namespace" },
+            { "width", "dimension" },
+            { "height", "dimension" },
         },
     }
 }
 
-statistics.register("metapost processing time", function()
-    local n =  metapost.n
+statistics.register("metapost", function()
+    local n = metapost.nofruns
     if n and n > 0 then
-        local nofconverted = metapost.makempy.nofconverted
         local elapsedtime = statistics.elapsedtime
-        local elapsed = statistics.elapsed
-        local instances, memory = metapost.getstatistics(true)
-        local str = format("%s seconds, loading: %s, execution: %s, n: %s, average: %s, instances: %i, memory: %0.3f M",
+        local elapsed     = statistics.elapsed
+        local instances,
+              memory      = metapost.getstatistics(true)
+        return format("%s seconds, loading: %s, execution: %s, n: %s, average: %s, instances: %i, luacalls: %i, memory: %0.3f M",
             elapsedtime(metapost), elapsedtime(mplib), elapsedtime(metapost.exectime), n,
             elapsedtime((elapsed(metapost) + elapsed(mplib) + elapsed(metapost.exectime)) / n),
-            instances, memory/(1024*1024))
-        if nofconverted > 0 then
-            return format("%s, external: %s (%s calls)",
-                str, elapsedtime(metapost.makempy), nofconverted)
-        else
-            return str
-        end
+            instances, metapost.nofscriptruns(),memory/(1024*1024))
     else
         return nil
     end
