@@ -11,7 +11,7 @@ if not modules then modules = { } end modules ['syst-aux'] = {
 -- utfmatch(str,"(.?)(.*)$")
 -- utf.sub(str,1,1)
 
-local tonumber = tonumber
+local tonumber, next = tonumber, next
 local utfsub = utf.sub
 local P, S, R, C, Cc, Cs, Carg, lpegmatch = lpeg.P, lpeg.S, lpeg.R, lpeg.C, lpeg.Cc, lpeg.Cs, lpeg.Carg, lpeg.match
 local next = next
@@ -23,7 +23,7 @@ local formatters        = string.formatters
 local setcatcode        = tex.setcatcode
 local utf8character     = lpeg.patterns.utf8character
 local settings_to_array = utilities.parsers.settings_to_array
-local settings_to_set  = utilities.parsers.settings_to_set
+local settings_to_set   = utilities.parsers.settings_to_set
 local setmacro          = interfaces.setmacro
 
 local pattern           = C(utf8character^-1) * C(P(1)^0)
@@ -61,7 +61,7 @@ local ctx_doifelse = commands.doifelse
 
 implement {
     name      = "doifelsefirstchar",
-    arguments = { "string", "string" },
+    arguments = "2 strings",
     actions   = function(str,chr)
         ctx_doifelse(lpegmatch(pattern,str) == chr)
     end
@@ -69,7 +69,7 @@ implement {
 
 implement {
     name      = "getsubstring",
-    arguments = { "string", "string", "string" },
+    arguments = "3 strings",
     actions   = function(str,first,last)
         context(utfsub(str,tonumber(first),tonumber(last)))
     end
@@ -323,9 +323,47 @@ end
 implement { name = "texdefinition_one", actions = texdefinition_one, scope = "private", arguments = "string" }
 implement { name = "texdefinition_two", actions = texdefinition_two, scope = "private" }
 
-implement { name = "upper", arguments = "string", actions = { utf.upper,    context } }
-implement { name = "lower", arguments = "string", actions = { utf.lower,    context } }
-implement { name = "strip", arguments = "string", actions = { string.strip, context } } -- or utf.strip
+do
+
+    -- Quite probably we don't yet have characters loaded so we delay some
+    -- aliases.
+
+    local _lower_, _upper_, _strip_
+
+    _lower_ = function(s)
+        if characters and characters.lower then
+            _lower_ = characters.lower
+            return _lower_(s)
+        end
+        return string.lower(s)
+    end
+
+    _upper_ = function(s)
+        if characters and characters.upper then
+            _upper_ = characters.upper
+            return _upper_(s)
+        end
+        return string.upper(s)
+    end
+
+    _strip_ = function(s)
+        -- or utf.strip
+        if string.strip then
+            _strip_ = string.strip
+            return _strip_(s)
+        end
+        return s
+    end
+
+    local function lower(s) context(_lower_(s)) end
+    local function upper(s) context(_upper_(s)) end
+    local function strip(s) context(_strip_(s)) end
+
+    implement { name = "upper", arguments = "string", actions = upper }
+    implement { name = "lower", arguments = "string", actions = lower }
+    implement { name = "strip", arguments = "string", actions = strip }
+
+end
 
 implement {
     name      = "converteddimen",
@@ -351,6 +389,13 @@ implement {
     end
 }
 
+implement {
+    name    = "benchmarktimer",
+    actions = function()
+        statistics.benchmarktimer("whatever")
+    end
+}
+
 implement {
     name    = "elapsedtime",
     actions = function()
@@ -373,10 +418,20 @@ implement {
 
 -- not faster but just less tracing:
 
-local firstoftwoarguments  = context.firstoftwoarguments
-local secondoftwoarguments = context.secondoftwoarguments
-local firstofoneargument   = context.firstofoneargument
-local gobbleoneargument    = context.gobbleoneargument
+local ctx_protected_cs         = context.protected.cs -- more efficient
+
+local ctx_firstoftwoarguments  = ctx_protected_cs.firstoftwoarguments
+local ctx_secondoftwoarguments = ctx_protected_cs.secondoftwoarguments
+local ctx_firstofoneargument   = ctx_protected_cs.firstofoneargument
+local ctx_gobbleoneargument    = ctx_protected_cs.gobbleoneargument
+
+context.firstoftwoarguments    = ctx_firstoftwoarguments
+context.secondoftwoarguments   = ctx_secondoftwoarguments
+context.firstofoneargument     = ctx_firstofoneargument
+context.gobbleoneargument      = ctx_gobbleoneargument
+
+local ctx_iftrue  = context.iftrue
+local ctx_iffalse = context.iffalse
 
 local hash = utilities.parsers.hashes.settings_to_set
 
@@ -384,14 +439,14 @@ local function doifelsecommon(a,b)
     if a == b then
         setmacro("commalistelement",a)
         if a == "" then
-            secondoftwoarguments()
+            ctx_secondoftwoarguments()
         else
-            firstoftwoarguments()
+            ctx_firstoftwoarguments()
         end
         return
     end
-    local ba = find(a,",")
-    local bb = find(b,",")
+    local ba = find(a,",",1,true)
+    local bb = find(b,",",1,true)
     if ba and bb then
         local ha = hash[a]
         local hb = hash[b]
@@ -400,7 +455,7 @@ local function doifelsecommon(a,b)
         for k in next, ha do
             if hb[k] then
                 setmacro("commalistelement",k)
-                firstoftwoarguments()
+                ctx_firstoftwoarguments()
                 return
             end
         end
@@ -408,33 +463,33 @@ local function doifelsecommon(a,b)
         if hash[a][b] then
      -- if settings_to_set(a)[b] then
             setmacro("commalistelement",b)
-            firstoftwoarguments()
+            ctx_firstoftwoarguments()
             return
         end
     elseif bb then
         if hash[b][a] then
      -- if settings_to_set(b)[a] then
             setmacro("commalistelement",a)
-            firstoftwoarguments()
+            ctx_firstoftwoarguments()
             return
         end
     end
     setmacro("commalistelement","")
-    secondoftwoarguments()
+    ctx_secondoftwoarguments()
 end
 
 local function doifcommon(a,b)
     if a == b then
         setmacro("commalistelement",a)
         if a == "" then
-            gobbleoneargument()
+            ctx_gobbleoneargument()
         else
-            firstofoneargument()
+            ctx_firstofoneargument()
         end
         return
     end
-    local ba = find(a,",")
-    local bb = find(b,",")
+    local ba = find(a,",",1,true)
+    local bb = find(b,",",1,true)
     if ba and bb then
         local ha = hash[a]
         local hb = hash[b]
@@ -443,7 +498,7 @@ local function doifcommon(a,b)
         for k in next, ha do
             if hb[k] then
                 setmacro("commalistelement",k)
-                firstofoneargument()
+                ctx_firstofoneargument()
                 return
             end
         end
@@ -451,33 +506,33 @@ local function doifcommon(a,b)
         if hash[a][b] then
      -- if settings_to_set(a)[b] then
             setmacro("commalistelement",b)
-            firstofoneargument()
+            ctx_firstofoneargument()
             return
         end
     elseif bb then
         if hash[b][a] then
      -- if settings_to_set(b)[a] then
             setmacro("commalistelement",a)
-            firstofoneargument()
+            ctx_firstofoneargument()
             return
         end
     end
     setmacro("commalistelement","")
-    gobbleoneargument()
+    ctx_gobbleoneargument()
 end
 
 local function doifnotcommon(a,b)
     if a == b then
         setmacro("commalistelement",a)
         if a == "" then
-            firstofoneargument()
+            ctx_firstofoneargument()
         else
-            gobbleoneargument()
+            ctx_gobbleoneargument()
         end
         return
     end
-    local ba = find(a,",")
-    local bb = find(b,",")
+    local ba = find(a,",",1,true)
+    local bb = find(b,",",1,true)
     if ba and bb then
         local ha = hash[a]
         local hb = hash[b]
@@ -486,7 +541,7 @@ local function doifnotcommon(a,b)
         for k in next, ha do
             if hb[k] then
                 setmacro("commalistelement",k)
-                gobbleoneargument()
+                ctx_gobbleoneargument()
                 return
             end
         end
@@ -494,129 +549,190 @@ local function doifnotcommon(a,b)
         if hash[a][b] then
      -- if settings_to_set(a)[b] then
             setmacro("commalistelement",b)
-            gobbleoneargument()
+            ctx_gobbleoneargument()
             return
         end
     elseif bb then
         if hash[b][a] then
      -- if settings_to_set(b)[a] then
             setmacro("commalistelement",a)
-            gobbleoneargument()
+            ctx_gobbleoneargument()
             return
         end
     end
     setmacro("commalistelement","")
-    firstofoneargument()
+    ctx_firstofoneargument()
 end
 
+-- local function hascommonargumentcondition(a,b)
+--     if a == b then
+--         setmacro("commalistelement",a)
+--         if a == "" then
+--             ctx_iffalse()
+--         else
+--             ctx_iftrue()
+--         end
+--         return
+--     end
+--     local ba = find(a,",",1,true)
+--     local bb = find(b,",",1,true)
+--     if ba and bb then
+--         local ha = hash[a]
+--         local hb = hash[b]
+--         for k in next, ha do
+--             if hb[k] then
+--                 setmacro("commalistelement",k)
+--                 ctx_iftrue()
+--                 return
+--             end
+--         end
+--     elseif ba then
+--         if hash[a][b] then
+--             setmacro("commalistelement",b)
+--             ctx_iftrue()
+--             return
+--         end
+--     elseif bb then
+--         if hash[b][a] then
+--             setmacro("commalistelement",a)
+--             ctx_iftrue()
+--             return
+--         end
+--     end
+--     setmacro("commalistelement","")
+--     ctx_iffalse()
+-- end
+
 local function doifelseinset(a,b)
     if a == b then
         setmacro("commalistelement",a)
         if a == "" then
-            secondoftwoarguments()
+            ctx_secondoftwoarguments()
         else
-            firstoftwoarguments()
+            ctx_firstoftwoarguments()
         end
         return
     end
-    local bb = find(b,",")
+    local bb = find(b,",",1,true)
     if bb then
         if hash[b][a] then
      -- if settings_to_set(b)[a] then
             setmacro("commalistelement",a)
-            firstoftwoarguments()
+            ctx_firstoftwoarguments()
             return
         end
     end
     setmacro("commalistelement","")
-    secondoftwoarguments()
+    ctx_secondoftwoarguments()
 end
 
 local function doifinset(a,b)
     if a == b then
         setmacro("commalistelement",a)
         if a == "" then
-            gobbleoneargument()
+            ctx_gobbleoneargument()
         else
-            firstofoneargument()
+            ctx_firstofoneargument()
         end
         return
     end
-    local bb = find(b,",")
+    local bb = find(b,",",1,true)
     if bb then
        if hash[b][a] then
     -- if settings_to_set(b)[a] then
             setmacro("commalistelement",a)
-            firstofoneargument()
+            ctx_firstofoneargument()
             return
         end
     end
     setmacro("commalistelement","")
-    gobbleoneargument()
+    ctx_gobbleoneargument()
 end
 
 local function doifnotinset(a,b)
     if a == b then
         setmacro("commalistelement",a)
         if a == "" then
-            firstofoneargument()
+            ctx_firstofoneargument()
         else
-            gobbleoneargument()
+            ctx_gobbleoneargument()
         end
         return
     end
-    local bb = find(b,",")
+    local bb = find(b,",",1,true)
     if bb then
         if hash[b][a] then
      -- if settings_to_set(b)[a] then
             setmacro("commalistelement",a)
-            gobbleoneargument()
+            ctx_gobbleoneargument()
             return
         end
     end
     setmacro("commalistelement","")
-    firstofoneargument()
+    ctx_firstofoneargument()
 end
 
 implement {
     name      = "doifelsecommon",
     actions   = doifelsecommon,
-    arguments = { "string", "string" },
+    arguments = "2 strings",
 }
 
 implement {
     name      = "doifcommon",
     actions   = doifcommon,
-    arguments = { "string", "string" },
+    arguments = "2 strings",
 }
 
 implement {
     name      = "doifnotcommon",
     actions   = doifnotcommon,
-    arguments = { "string", "string" },
+    arguments = "2 strings",
 }
 
+-- implement {
+--     name      = "hascommonargumentcondition",
+--     actions   = hascommonargumentcondition,
+--     arguments = "2 strings",
+--     arguments = { "argument", "argument" },
+-- }
+
 implement {
     name      = "doifelseinset",
     actions   = doifelseinset,
-    arguments = { "string", "string" },
+    arguments = "2 strings",
+--     arguments = { "argument", "argument" },
 }
 
 implement {
     name      = "doifinset",
     actions   = doifinset,
-    arguments = { "string", "string" },
+    arguments = "2 strings",
 }
 
 implement {
     name      = "doifnotinset",
     actions   = doifnotinset,
-    arguments = { "string", "string" },
+    arguments = "2 strings",
 }
 
+-- done elsewhere:
+--
+-- local function firstinset(a)
+--     local aa = hash[a]
+--     context(aa and aa[1] or a)
+-- end
+--
+-- implement {
+--     name      = "firstinset",
+--     actions   = firstinset,
+--     arguments = "string",
+--     private   = false,
+-- }
+
 -- implement {
 --     name      = "stringcompare",
---     arguments = { "string", "string" },
+--     arguments = "2 strings",
 --     actions   = function(a,b)
 --         context((a == b and 0) or (a > b and 1) or -1)
 --     end
@@ -624,7 +740,7 @@ implement {
 --
 -- implement {
 --     name      = "doifelsestringafter",
---     arguments = { "string", "string" },
+--     arguments = "2 strings",
 --     actions   = function(a,b)
 --         ctx_doifelse((a == b and 0) or (a > b and 1) or -1)
 --     end
@@ -632,7 +748,7 @@ implement {
 --
 -- implement {
 --     name      = "doifelsestringbefore",
---     arguments = { "string", "string" },
+--     arguments = "2 strings",
 --     actions   = function(a,b)
 --         ctx_doifelse((a == b and 0) or (a < b and -1) or 1)
 --     end
@@ -640,7 +756,7 @@ implement {
 
 -- implement { -- not faster than addtocommalist
 --     name      = "additemtolist", -- unique
---     arguments = { "string", "string" },
+--     arguments = "2 strings",
 --     actions   = function(l,s)
 --         if l == "" or s == l then
 --          -- s = s
@@ -652,3 +768,18 @@ implement {
 --         context(s)
 --     end
 -- }
+
+local bp = number.dimenfactors.bp
+
+interfaces.implement {
+    name      = "tobigpoints",
+    actions   = function(d) context("%.5F",bp * d) end,
+    arguments = "dimension",
+}
+
+interfaces.implement {
+    name      = "towholebigpoints",
+    actions   = function(d) context("%r",bp * d) end,
+    arguments = "dimension",
+}
+
