@@ -26,7 +26,7 @@ this mechamism will be improved so that it can replace its older cousin.
 -- todo: use linked list instead of r/c array
 -- todo: we can use the sum of previously forced widths for column spans
 
-local tonumber, next, rawget = tonumber, next, rawget
+local tonumber, next = tonumber, next
 
 local commands            = commands
 local context             = context
@@ -70,7 +70,7 @@ local getbox              = nuts.getbox
 local getwhd              = nuts.getwhd
 
 local setlink             = nuts.setlink
-local setdir              = nuts.setdir
+local setdirection        = nuts.setdirection
 local setshift            = nuts.setshift
 
 local copy_node_list      = nuts.copy_list
@@ -84,6 +84,8 @@ local new_glue            = nodepool.glue
 local new_kern            = nodepool.kern
 local new_hlist           = nodepool.hlist
 
+local lefttoright_code    = nodes.dirvalues.lefttoright
+
 local v_stretch           = variables.stretch
 local v_normal            = variables.normal
 local v_width             = variables.width
@@ -375,7 +377,8 @@ function xtables.set_reflow_width()
     --
     drc.dimensionstate = dimensionstate
     --
-    local nx, ny = drc.nx, drc.ny
+    local nx = drc.nx
+    local ny = drc.ny
     if nx > 1 or ny > 1 then
      -- local spans = data.spans -- not used
         local self = true
@@ -475,36 +478,50 @@ end
 function xtables.initialize_construct()
     local r = data.currentrow
     local c = data.currentcolumn + 1
+    local settings = data.settings
     local rows = data.rows
     local row = rows[r]
     while row[c].span do -- can also be previous row ones
         c = c + 1
     end
     data.currentcolumn = c
-    local widths  = data.widths
-    local heights = data.heights
-    local depths  = data.depths
+    local widths    = data.widths
+    local heights   = data.heights
+    local depths    = data.depths
+    local distances = data.distances
     --
     local drc = row[c]
     local wd  = drc.wd
     local ht  = drc.ht
     local dp  = drc.dp
+    local nx  = drc.nx - 1
+    local ny  = drc.ny - 1
     --
     local width  = widths[c]
     local height = heights[r]
     local depth  = depths[r] -- problem: can be the depth of a one liner
     --
-    for x=1,drc.nx-1 do
-        width = width + widths[c+x]
+    local total  = height + depth
+    --
+    if nx > 0 then
+        for x=1,nx do
+            width = width + widths[c+x] + distances[c+x-1]
+        end
+        local distance = settings.columndistance
+        if distance ~= 0 then
+            width = width + nx * distance
+        end
     end
     --
-    local total = height + depth
-    local ny = drc.ny
-    if ny > 1 then
-        for y=1,ny-1 do
+    if ny > 0 then
+        for y=1,ny do
             local nxt = r + y
             total = total + heights[nxt] + depths[nxt]
         end
+        local distance = settings.rowdistance
+        if distance ~= 0 then
+            total = total + ny * distance
+        end
     end
     --
     texsetdimen("d_tabl_x_width",width)
@@ -886,13 +903,13 @@ function xtables.construct()
                 -- we have a direction issue here but hpack_node_list(list,0,"exactly","TLT") cannot be used
                 -- due to the fact that we need the width
                 local hbox = hpack_node_list(list)
-                setdir(hbox,"TLT")
+                setdirection(hbox,lefttoright_code)
                 result[nofr] = {
                     hbox,
                     size,
                     i < nofrange and rowdistance > 0 and rowdistance or false, -- might move
                     false,
-                    rp and rp.samepage or false,
+                    rp or false,
                 }
             end
         end
@@ -901,7 +918,7 @@ function xtables.construct()
             result[1]   [5] = false
             result[nofr][5] = false
             for i=2,nofr-1 do
-                local r = result[i]
+                local r = result[i][5]
                 if r == v_both or r == v_before then
                     result[i-1][5] = true
                 elseif r == v_after then
