@@ -10,7 +10,6 @@ if not modules then modules = { } end modules ['trac-log'] = {
 -- terminal handler in lua then. Ok, maybe it's slower then, so a no-go.
 
 local next, type, select, print = next, type, select, print
-local write_nl, write = texio and texio.write_nl or print, texio and texio.write or io.write
 local format, gmatch, find = string.format, string.gmatch, string.find
 local concat, insert, remove = table.concat, table.insert, table.remove
 local topattern = string.topattern
@@ -18,10 +17,17 @@ local utfchar = utf.char
 local datetime = os.date
 local openfile = io.open
 
+local runningtex = tex and (tex.jobname or tex.formatname)
+-- local write_nl = texio and texio.write_nl or print
+-- local write    = texio and texio.write    or io.write
+
+local write_nl = runningtex and texio and texio.write_nl or print
+local write    = runningtex and texio and texio.write    or io.write
+
 local setmetatableindex = table.setmetatableindex
 local formatters        = string.formatters
-
-local texgetcount       = tex and tex.getcount
+local settings_to_hash  = utilities.parsers.settings_to_hash
+local sortedkeys        = table.sortedkeys
 
 -- variant is set now
 
@@ -108,17 +114,34 @@ local direct, subdirect, writer, pushtarget, poptarget, setlogfile, settimedlog,
 -- we don't want this overhead for single messages (not that there are that
 -- many; we could have a special weak table)
 
-if tex and (tex.jobname or tex.formatname) then
+local function ansisupported(specification)
+    if specification ~= "ansi" and specification ~= "ansilog" then
+        return false
+    elseif os and os.enableansi then
+        return os.enableansi()
+    else
+        return false
+    end
+end
+
+if runningtex and texio then
 
     if texio.setescape then
         texio.setescape(0) -- or (false)
     end
 
-    if arg then
+    if arg and ansisupported then
         -- we're don't have environment.arguments yet
         for k, v in next, arg do -- k can be negative !
             if v == "--ansi" or v == "--c:ansi" then
-                variant = "ansi"
+                if ansisupported("ansi") then
+                    variant = "ansi"
+                end
+                break
+            elseif v == "--ansilog" or v == "--c:ansilog" then
+                if ansisupported("ansilog") then
+                    variant = "ansilog"
+                end
                 break
             end
         end
@@ -240,6 +263,11 @@ if tex and (tex.jobname or tex.formatname) then
         }
     }
 
+    variants.ansilog = {
+        formats = variants.ansi.formats,
+        targets = variants.default.targets,
+    }
+
     logs.flush = io.flush
 
     writer = function(...)
@@ -362,6 +390,9 @@ if tex and (tex.jobname or tex.formatname) then
             t = specification.targets
             f = specification.formats or specification
         else
+            if not ansisupported(specification) then
+                specification = "default"
+            end
             local v = variants[specification]
             if v then
                 t = v.targets
@@ -388,8 +419,8 @@ if tex and (tex.jobname or tex.formatname) then
         subdirect_nop = f.subdirect_nop
         status_yes    = f.status_yes
         status_nop    = f.status_nop
-        if variant == "ansi" then
-            useluawrites() -- because tex escapes ^^
+        if variant == "ansi" or variant == "ansilog" then
+            useluawrites() -- because tex escapes ^^, not needed in lmtx
         end
         settarget(whereto)
     end
@@ -399,6 +430,15 @@ if tex and (tex.jobname or tex.formatname) then
     setlogfile  = ignore
     settimedlog = ignore
 
+ -- settimedlog = function()
+ --     local localtime = os.localtime
+ --     local writeline = write_nl
+ --     write_nl = function(f,...)
+ --         writeline(f,localtime() .. " | " .. concat { ... })
+ --     end
+ --     settimedlog = ignore
+ -- end
+
 else
 
     local report_yes, subreport_yes, status_yes
@@ -496,6 +536,9 @@ else
             if type(specification) == "table" then
                 f = specification.formats or specification
             else
+                if not ansisupported(specification) then
+                    specification = "default"
+                end
                 local v = variants[specification]
                 if v then
                     f = v.formats
@@ -575,15 +618,15 @@ logs.newline         = newline
 
 -- todo: renew (un) locks when a new one is added and wildcard
 
-local data, states = { }, nil
+local data   = { }
+local states = nil
+local force  = false
 
 function logs.reporter(category,subcategory)
     local logger = data[category]
     if not logger then
-        local state = false
-        if states == true then
-            state = true
-        elseif type(states) == "table" then
+        local state = states == true
+        if not state and type(states) == "table" then
             for c, _ in next, states do
                 if find(category,c) then
                     state = true
@@ -593,7 +636,7 @@ function logs.reporter(category,subcategory)
         end
         logger = {
             reporters = { },
-            state = state,
+            state     = state,
         }
         data[category] = logger
     end
@@ -601,7 +644,7 @@ function logs.reporter(category,subcategory)
     if not reporter then
         if subcategory then
             reporter = function(...)
-                if not logger.state then
+                if force or not logger.state then
                     subreport(category,subcategory,...)
                 end
             end
@@ -609,7 +652,7 @@ function logs.reporter(category,subcategory)
         else
             local tag = category
             reporter = function(...)
-                if not logger.state then
+                if force or not logger.state then
                     report(category,...)
                 end
             end
@@ -645,7 +688,7 @@ end
 -- so far
 
 local function setblocked(category,value) -- v.state == value == true : disable
-    if category == true then
+    if category == true or category == "all" then
         -- lock all
         category, value = "*", true
     elseif category == false then
@@ -661,7 +704,8 @@ local function setblocked(category,value) -- v.state == value == true : disable
             v.state = value
         end
     else
-        states = utilities.parsers.settings_to_hash(category,type(states)=="table" and states or nil)
+        alllocked = false
+        states = settings_to_hash(category,type(states)=="table" and states or nil)
         for c in next, states do
             local v = data[c]
             if v then
@@ -687,7 +731,7 @@ function logs.enable(category)
 end
 
 function logs.categories()
-    return table.sortedkeys(data)
+    return sortedkeys(data)
 end
 
 function logs.show()
@@ -710,7 +754,7 @@ function logs.show()
                 max = m
             end
         end
-        local subcategories = concat(table.sortedkeys(reporters),", ")
+        local subcategories = concat(sortedkeys(reporters),", ")
         if state == true then
             state = "disabled"
         elseif state == false then
@@ -746,59 +790,52 @@ end)
 
 -- tex specific loggers (might move elsewhere)
 
-local report_pages = logs.reporter("pages") -- not needed but saves checking when we grep for it
+if tex then
 
-local real, user, sub
+    local report      = logs.reporter("pages") -- not needed but saves checking when we grep for it
+    local texgetcount = tex and tex.getcount
 
-function logs.start_page_number()
-    real = texgetcount("realpageno")
-    user = texgetcount("userpageno")
-    sub  = texgetcount("subpageno")
-end
+    local real, user, sub = 0, 0, 0
 
-local timing    = false
-local starttime = nil
-local lasttime  = nil
-
-trackers.register("pages.timing", function(v) -- only for myself (diagnostics)
-    starttime = os.clock()
-    timing    = true
-end)
-
-function logs.stop_page_number() -- the first page can includes the initialization so we omit this in average
-    if timing then
-        local elapsed, average
-        local stoptime = os.clock()
-        if not lasttime or real < 2 then
-            elapsed   = stoptime
-            average   = stoptime
-            starttime = stoptime
-        else
-            elapsed  = stoptime - lasttime
-            average  = (stoptime - starttime) / (real - 1)
-        end
-        lasttime = stoptime
-        if real <= 0 then
-            report_pages("flushing page, time %0.04f / %0.04f",elapsed,average)
-        elseif user <= 0 then
-            report_pages("flushing realpage %s, time %0.04f / %0.04f",real,elapsed,average)
-        elseif sub <= 0 then
-            report_pages("flushing realpage %s, userpage %s, time %0.04f / %0.04f",real,user,elapsed,average)
-        else
-            report_pages("flushing realpage %s, userpage %s, subpage %s, time %0.04f / %0.04f",real,user,sub,elapsed,average)
-        end
-    else
-        if real <= 0 then
-            report_pages("flushing page")
-        elseif user <= 0 then
-            report_pages("flushing realpage %s",real)
-        elseif sub <= 0 then
-            report_pages("flushing realpage %s, userpage %s",real,user)
-        else
-            report_pages("flushing realpage %s, userpage %s, subpage %s",real,user,sub)
-        end
+    function logs.start_page_number()
+        real = texgetcount("realpageno")
+        user = texgetcount("userpageno")
+        sub  = texgetcount("subpageno")
     end
-    logs.flush()
+
+    local timing   = false
+    local lasttime = nil
+
+    trackers.register("pages.timing", function(v) -- only for myself (diagnostics)
+        timing = ""
+    end)
+
+    function logs.stop_page_number() -- the first page can includes the initialization so we omit this in average
+        if timing then
+            local elapsed = statistics.currenttime(statistics)
+            local average, page
+            if not lasttime or real < 2 then
+                average = elapsed
+                page    = elapsed
+            else
+                average = elapsed / (real - 1)
+                page    = elapsed - lasttime
+            end
+            lasttime = elapsed
+            timing   = formatters[", total %0.03f, page %0.03f, average %0.03f"](elapsed,page,average)
+        end
+        if real <= 0 then
+            report("flushing page%s",timing)
+        elseif user <= 0 then
+            report("flushing realpage %s%s",real,timing)
+        elseif sub <= 0 then
+            report("flushing realpage %s, userpage %s%s",real,user,timing)
+        else
+            report("flushing realpage %s, userpage %s, subpage %s%s",real,user,sub,timing)
+        end
+        logs.flush()
+    end
+
 end
 
 -- we don't have show_open and show_close callbacks yet
@@ -808,38 +845,6 @@ local nesting      = 0
 local verbose      = false
 local hasscheme    = url.hasscheme
 
-function logs.show_open(name)
- -- if hasscheme(name) ~= "virtual" then
- --     if verbose then
- --         nesting = nesting + 1
- --         report_files("level %s, opening %s",nesting,name)
- --     else
- --         write(formatters["(%s"](name)) -- tex adds a space
- --     end
- -- end
-end
-
-function logs.show_close(name)
- -- if hasscheme(name) ~= "virtual" then
- --     if verbose then
- --         report_files("level %s, closing %s",nesting,name)
- --         nesting = nesting - 1
- --     else
- --         write(")") -- tex adds a space
- --     end
- -- end
-end
-
-function logs.show_load(name)
- -- if hasscheme(name) ~= "virtual" then
- --     if verbose then
- --         report_files("level %s, loading %s",nesting+1,name)
- --     else
- --         write(formatters["(%s)"](name))
- --     end
- -- end
-end
-
 -- there may be scripts out there using this:
 
 local simple = logs.reporter("comment")
@@ -937,6 +942,15 @@ logs.reporters = reporters
 logs.exporters = exporters
 
 function logs.application(t)
+    --
+    local arguments = environment and environment.arguments
+    if arguments then
+        local ansi = arguments.ansi or arguments.ansilog
+        if ansi then
+            logs.setformatters(arguments.ansi and "ansi" or "ansilog")
+        end
+    end
+    --
     t.name     = t.name   or "unknown"
     t.banner   = t.banner
     t.moreinfo = moreinfo
@@ -1019,7 +1033,7 @@ end
 
 if tex and tex.error then
     function logs.texerrormessage(...) -- for the moment we put this function here
-        tex.error(format(...), { })
+        tex.error(format(...))
     end
 else
     function logs.texerrormessage(...)
@@ -1030,8 +1044,8 @@ end
 -- this is somewhat slower but prevents out-of-order messages when print is mixed
 -- with texio.write
 
-io.stdout:setvbuf('no')
-io.stderr:setvbuf('no')
+-- io.stdout:setvbuf('no')
+-- io.stderr:setvbuf('no')
 
 -- windows: > nul  2>&1
 -- unix   : > null 2>&1
@@ -1039,3 +1053,104 @@ io.stderr:setvbuf('no')
 if package.helpers.report then
     package.helpers.report = logs.reporter("package loader") -- when used outside mtxrun
 end
+
+if tex then
+
+    local finalactions  = { }
+    local fatalerrors   = { }
+    local possiblefatal = { }
+    local loggingerrors = false
+
+    function logs.loggingerrors()
+        return loggingerrors
+    end
+
+    directives.register("logs.errors",function(v)
+        loggingerrors = v
+        if type(v) == "string" then
+            fatalerrors = settings_to_hash(v)
+        else
+            fatalerrors = { }
+        end
+    end)
+
+    function logs.registerfinalactions(...)
+        insert(finalactions,...) -- so we can force an order if needed
+    end
+
+    local what   = nil
+    local report = nil
+    local state  = nil
+    local target = nil
+
+    local function startlogging(t,r,w,s)
+        target = t
+        state  = force
+        force  = true
+        report = type(r) == "function" and r or logs.reporter(r)
+        what   = w
+        pushtarget(target)
+        newline()
+        if s then
+            report("start %s: %s",what,s)
+        else
+            report("start %s",what)
+        end
+        if target == "logfile" then
+            newline()
+        end
+        return report
+    end
+
+    local function stoplogging()
+        if target == "logfile" then
+            newline()
+        end
+        report("stop %s",what)
+        if target == "logfile" then
+            newline()
+        end
+        poptarget()
+        state = oldstate
+    end
+
+    function logs.startfilelogging(...)
+        return startlogging("logfile", ...)
+    end
+
+    logs.stopfilelogging = stoplogging
+
+    local done = false
+
+    function logs.starterrorlogging(r,w,...)
+        if not done then
+            pushtarget("terminal")
+            newline()
+            logs.report("error logging","start possible issues")
+            poptarget()
+            done = true
+        end
+        if fatalerrors[w] then
+            possiblefatal[w] = true
+        end
+        return startlogging("terminal",r,w,...)
+    end
+
+    logs.stoperrorlogging = stoplogging
+
+    function logs.finalactions()
+        if #finalactions > 0 then
+            for i=1,#finalactions do
+                finalactions[i]()
+            end
+            if done then
+                pushtarget("terminal")
+                newline()
+                logs.report("error logging","stop possible issues")
+                poptarget()
+            end
+            return next(possiblefatal) and sortedkeys(possiblefatal) or false
+        end
+    end
+
+end
