@@ -9,8 +9,9 @@ if not modules then modules = { } end modules ['trac-set'] = { -- might become u
 -- maybe this should be util-set.lua
 
 local type, next, tostring, tonumber = type, next, tostring, tonumber
+local print = print
 local concat, sortedhash = table.concat, table.sortedhash
-local format, find, lower, gsub, topattern = string.format, string.find, string.lower, string.gsub, string.topattern
+local formatters, find, lower, gsub, topattern = string.formatters, string.find, string.lower, string.gsub, string.topattern
 local is_boolean = string.is_boolean
 local settings_to_hash = utilities.parsers.settings_to_hash
 local allocate = utilities.storage.allocate
@@ -30,11 +31,11 @@ local data        = { }
 -- The sorting is needed to get a predictable setters in case of *.
 
 local trace_initialize = false -- only for testing during development
+local frozen           = true  -- this needs checking
 
 function setters.initialize(filename,name,values) -- filename only for diagnostics
     local setter = data[name]
     if setter then
-        frozen = true -- don't permitoverload
      -- trace_initialize = true
         local data = setter.data
         if data then
@@ -254,8 +255,8 @@ end
 
 local enable, disable, register, list, show = setters.enable, setters.disable, setters.register, setters.list, setters.show
 
-function setters.report(setter,...)
-    print(format("%-15s : %s\n",setter.name,format(...)))
+function setters.report(setter,fmt,...)
+    print(formatters["%-15s : %s\n"](setter.name,formatters[fmt](...)))
 end
 
 local function default(setter,name)
@@ -278,7 +279,7 @@ function setters.new(name) -- we could use foo:bar syntax (but not used that oft
         disable  = function(...)         disable (setter,...) end,
         reset    = function(...)         reset   (setter,...) end, -- can be dangerous
         register = function(...)         register(setter,...) end,
-        list     = function(...)         list    (setter,...) end,
+        list     = function(...)  return list    (setter,...) end,
         show     = function(...)         show    (setter,...) end,
         default  = function(...)  return default (setter,...) end,
         value    = function(...)  return value   (setter,...) end,
