@@ -41,13 +41,11 @@ if not modules then modules = { } end modules ['typo-dha'] = {
 
 local nodes, node = nodes, node
 
-local trace_directions   = false  trackers.register("typesetters.directions.default", function(v) trace_directions = v end)
+local trace_directions   = false  trackers.register("typesetters.directions", function(v) trace_directions = v end)
 
 local report_directions  = logs.reporter("typesetting","text directions")
 
 local nuts               = nodes.nuts
-local tonut              = nuts.tonut
-local tonode             = nuts.tonode
 
 local getnext            = nuts.getnext
 local getprev            = nuts.getprev
@@ -55,25 +53,25 @@ local getchar            = nuts.getchar
 local getid              = nuts.getid
 local getsubtype         = nuts.getsubtype
 local getlist            = nuts.getlist
-local getfield           = nuts.getfield
 local getattr            = nuts.getattr
 local getprop            = nuts.getprop
-local getdir            = nuts.getdir
+local getdirection       = nuts.getdirection
 local isglyph            = nuts.isglyph -- or ischar
 
-local setfield           = nuts.setfield
 local setprop            = nuts.setprop
+local setstate           = nuts.setstate
 local setchar            = nuts.setchar
 
 local insert_node_before = nuts.insert_before
 local insert_node_after  = nuts.insert_after
 local remove_node        = nuts.remove
 local end_of_math        = nuts.end_of_math
+local start_of_par       = nuts.start_of_par
 
 local nodepool           = nuts.pool
 
 local nodecodes          = nodes.nodecodes
-local skipcodes          = nodes.skipcodes
+local gluecodes          = nodes.gluecodes
 
 local glyph_code         = nodecodes.glyph
 local math_code          = nodecodes.math
@@ -82,12 +80,14 @@ local glue_code          = nodecodes.glue
 local dir_code           = nodecodes.dir
 local localpar_code      = nodecodes.localpar
 
-local parfillskip_code   = skipcodes.parfillskip
+local dirvalues          = nodes.dirvalues
+local lefttoright_code   = dirvalues.lefttoright
+local righttoleft_code   = dirvalues.righttoleft
 
-local new_textdir        = nodepool.textdir
+local parfillskip_code   = gluecodes.parfillskip
+
+local new_direction      = nodepool.direction
 
-local hasbit             = number.hasbit
-local formatters         = string.formatters
 local insert             = table.insert
 
 local fonthashes         = fonts.hashes
@@ -101,21 +101,20 @@ local directions         = typesetters.directions
 local setcolor           = directions.setcolor
 local getglobal          = directions.getglobal
 
-local a_state            = attributes.private('state')
 local a_directions       = attributes.private('directions')
 
 local strip              = false
 
 local s_isol             = fonts.analyzers.states.isol
 
-local function stopdir(finish)
-    local n = new_textdir(finish == "TRT" and "-TRT" or "-TLT")
+local function stopdir(finish) -- we could use finish directly
+    local n = new_direction(finish == righttoleft_code and righttoleft_code or lefttoright_code,true)
     setprop(n,"direction",true)
     return n
 end
 
-local function startdir(finish)
-    local n = new_textdir(finish == "TRT" and "+TRT" or "+TLT")
+local function startdir(finish) -- we could use finish directly
+    local n = new_direction(finish == righttoleft_code and righttoleft_code or lefttoright_code)
     setprop(n,"direction",true)
     return n
 end
@@ -140,7 +139,7 @@ end
 
 local function process(start)
 
-    local head     = tonut(start) -- we have a global head
+    local head     = start
     local current  = head
     local autodir  = 0
     local embedded = 0
@@ -192,7 +191,7 @@ local function process(start)
                             end
                         elseif lro or override < 0 then
                             if direction == "r" or direction == "al" then
-                                setprop(current,a_state,s_isol) -- hm
+                                setstate(current,s_isol) -- hm
                                 direction = "l"
                                 reversed  = true
                             end
@@ -313,27 +312,31 @@ local function process(start)
             elseif id == kern_code then
                 setprop(current,"direction",'k')
             elseif id == dir_code then
-                local dir = getdir(current)
-                if dir == "+TRT" then
-                    autodir = -1
-                elseif dir == "+TLT" then
-                    autodir = 1
-                elseif dir == "-TRT" or dir == "-TLT" then
-                    if embedded and embedded~= 0 then
+                local direction, pop = getdirection(current)
+                if direction == righttoleft_code then
+                    if not pop then
+                        autodir = -1
+                    elseif embedded and embedded~= 0 then
+                        autodir = embedded
+                    else
+                        autodir = 0
+                    end
+                elseif direction == lefttoright_code then
+                    if not pop then
+                        autodir = 1
+                    elseif embedded and embedded~= 0 then
                         autodir = embedded
                     else
                         autodir = 0
                     end
-                else
-                    -- message
                 end
                 textdir = autodir
                 setprop(current,"direction",true)
-            elseif id == localpar_code then
-                local dir = getdir(current)
-                if dir == 'TRT' then
+            elseif id == localpar_code and start_of_par(current) then
+                local direction = getdirection(current)
+                if direction == righttoleft_code then
                     autodir = -1
-                elseif dir == 'TLT' then
+                elseif direction == lefttoright_code then
                     autodir = 1
                 end
                 pardir  = autodir
@@ -447,7 +450,7 @@ local function process(start)
         end
     end
 
-    return tonode(head), done
+    return head
 
 end
 
